from django.test import TestCase

from ..models import Message, Recipient, Contact
from ..compat import User, truncate_words


class MessageContactTests(TestCase):
    fixtures = ['users.json', 'messages.json']

    def test_string_formatting(self):
        """ Test the human representation of a message """
        contact = Contact.objects.get(pk=1)
        correct_format = "thoas and ampelmann"
        self.failUnlessEqual(contact.__unicode__(),
                             correct_format)

    def test_opposite_user(self):
        """ Test if the opposite user is returned """
        contact = Contact.objects.get(pk=1)
        thoas = User.objects.get(pk=1)
        ampelmann = User.objects.get(pk=2)

        # Test the opposites
        self.failUnlessEqual(contact.opposite_user(thoas),
                             ampelmann)

        self.failUnlessEqual(contact.opposite_user(ampelmann),
                             thoas)


class MessageModelTests(TestCase):
    fixtures = ['users.json', 'messages.json']

    def test_string_formatting(self):
        """ Test the human representation of a message """
        message = Message.objects.get(pk=1)
        truncated_body = truncate_words(message.body, 10)
        self.failUnlessEqual(message.__unicode__(),
                             truncated_body)


class MessageRecipientModelTest(TestCase):
    fixtures = ['users.json', 'messages.json']

    def test_string_formatting(self):
        """ Test the human representation of a recipient """
        recipient = Recipient.objects.get(pk=1)

        valid_unicode = '%s' % (recipient.discussion)

        self.failUnlessEqual(recipient.__unicode__(),
                             valid_unicode)

    def test_new(self):
        """ Test if the message that is new is correct """
        new_message = Recipient.objects.get(pk=1)
        read_message = Recipient.objects.get(pk=2)

        self.assertFalse(new_message.is_read())
        self.assertTrue(read_message.is_read())



from django.conf.urls import patterns, url

from . import views, settings
from .utils import load_class

pre_filter = load_class(settings.PRE_FILTER)


urlpatterns = patterns(
    '',
    url(r'^compose(?:\/(?P<recipients>[\w\-\_\+]+))?/$',
        pre_filter(views.MessageComposeView.as_view()),
        name='discussions_compose'),

    url(r'^view/(?P<discussion_id>[\d]+)/$',
        pre_filter(views.DiscussionDetailView.as_view()),
        name='discussions_detail'),

    url(r'^remove/$',
        pre_filter(views.DiscussionRemoveView.as_view()),
        name='discussions_remove'),

    url(r'^unremove/$',
        pre_filter(views.DiscussionRemoveView.as_view()),
        {'undo': True},
        name='discussions_unremove'),

    url(r'^(?:(?P<username>[\w\-\_]+))?$',
        pre_filter(views.DiscussionListView.as_view()),
        name='discussions_list'),

    url(r'^sent/$',
        pre_filter(views.DiscussionSentView.as_view()),
        name='discussions_sent'),

    url(r'^unread/$',
        pre_filter(views.DiscussionUnreadView.as_view()),
        name='discussions_unread'),

    url(r'^trash/$',
        pre_filter(views.DiscussionDeletedView.as_view()),
        name='discussions_deleted'),

    url(r'^move/(?P<folder_id>[\d]+)$',
        pre_filter(views.DiscussionMoveView.as_view()),
        name='discussions_move'),

    url(r'^read/$',
        pre_filter(views.DiscussionReadView.as_view()),
        name='discussions_read'),

    url(r'^folder/(?P<folder_id>[\d]+)$',
        pre_filter(views.FolderDetailView.as_view()),
        name='discussions_folder_detail'),

    url(r'^folder/(?P<folder_id>[\d]+)/update$',
        pre_filter(views.FolderUpdateView.as_view()),
        name='discussions_folder_update'),

    url(r'^folder/create/$',
        pre_filter(views.FolderCreateView.as_view()),
        name='discussions_folder_create'),
)



from django import forms
from django.utils.translation import ugettext_lazy as _

from ..fields import CommaSeparatedUserField
from ..models import Discussion, Folder


class ComposeForm(forms.Form):
    to = CommaSeparatedUserField(label=_('To'))
    subject = forms.CharField(label=_('Subject'),
                              widget=forms.TextInput(),
                              required=True)
    body = forms.CharField(label=_('Message'),
                           widget=forms.Textarea({'class': 'message'}),
                           required=True)

    def save(self, sender):
        """
        Save the discussion and send it out into the wide world.

        :param sender:
            The :class:`User` that sends the message.

        :return: The saved :class:`Discussion`.

        """
        to_user_list = self.cleaned_data['to']
        subject = self.cleaned_data['subject']
        body = self.cleaned_data['body']

        self.discussion = Discussion.objects.send_message(sender,
                                                          to_user_list,
                                                          subject,
                                                          body)

        return self.discussion


class ReplyForm(forms.Form):
    body = forms.CharField(label=_('Message'),
                           widget=forms.Textarea({'class': 'message'}),
                           required=True)

    def __init__(self, *args, **kwargs):
        self.discussion = kwargs.pop('discussion')

        super(ReplyForm, self).__init__(*args, **kwargs)

    def save(self, sender):
        return self.discussion.add_message(self.cleaned_data['body'], sender)


class FolderForm(forms.ModelForm):
    class Meta:
        exclude = ('user', 'discussions', 'created_at')
        model = Folder

    def save(self, user):
        self.instance.user = user

        return super(FolderForm, self).save()



from datetime import datetime

from django.db import models
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import python_2_unicode_compatible

from discussions.managers import (DiscussionManager, ContactManager,
                                  RecipientManager, MessageManager)

from ..utils import tznow

from model_utils import Choices


AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


@python_2_unicode_compatible
class Contact(models.Model):
    """
    Contact model.

    A contact is a user to whom a user has send a message to or
    received a message from.

    """
    from_user = models.ForeignKey(AUTH_USER_MODEL,
                                  verbose_name=_('from user'),
                                  related_name=('from_contact_users'))

    to_user = models.ForeignKey(AUTH_USER_MODEL,
                                verbose_name=_('to user'),
                                related_name=('to_contact_users'))

    latest_discussion = models.ForeignKey('Discussion',
                                          verbose_name=_("latest discussion"))

    objects = ContactManager()

    class Meta:
        unique_together = ('from_user', 'to_user')
        ordering = ['latest_discussion']
        verbose_name = _('contact')
        verbose_name_plural = _('contacts')
        app_label = 'discussions'

    def __str__(self):
        return (_('%(from_user)s and %(to_user)s')
                % {'from_user': self.from_user.username,
                   'to_user': self.to_user.username})

    def opposite_user(self, user):
        """
        Returns the user opposite of the user that is given

        :param user:
            A Django :class:`User`.

        :return:
            A Django :class:`User`.

        """
        if self.from_user == user:
            return self.to_user

        return self.from_user


class Recipient(models.Model):
    """
    Intermediate model to allow per recipient marking as
    deleted, read etc. of a message.

    """
    STATUS = Choices((0, 'read', _('read')),
                     (1, 'unread', _('unread')),
                     (2, 'deleted', _('deleted')))

    user = models.ForeignKey(AUTH_USER_MODEL,
                             verbose_name=_('recipient'))

    discussion = models.ForeignKey('Discussion',
                                   verbose_name=_('discussion'))

    folder = models.ForeignKey('Folder',
                               verbose_name=_('folder'),
                               null=True, blank=True,
                               related_name='recipients')

    read_at = models.DateTimeField(_('read at'),
                                   null=True,
                                   blank=True)

    deleted_at = models.DateTimeField(_('recipient deleted at'),
                                      null=True,
                                      blank=True)

    status = models.PositiveSmallIntegerField(choices=STATUS,
                                              default=STATUS.unread,
                                              verbose_name=_('Status'),
                                              db_index=True)

    objects = RecipientManager()

    class Meta:
        verbose_name = _('recipient')
        verbose_name_plural = _('recipients')
        app_label = 'discussions'

    def __unicode__(self):
        return (_('%(discussion)s')
                % {'discussion': self.discussion})

    def is_read(self):
        """ Returns a boolean whether the recipient has read the message """
        return self.status == self.STATUS.read

    def is_deleted(self):
        """ Returns a boolean whether the recipient has deleted the message """
        return self.status == self.STATUS.deleted

    def mark_as_deleted(self, commit=True):
        self.deleted_at = datetime.now()
        self.status = self.STATUS.deleted

        if commit:
            self.save()

    def mark_as_read(self, commit=True):
        self.read_at = datetime.now()
        self.status = self.STATUS.read

        if commit:
            self.save()


@python_2_unicode_compatible
class Discussion(models.Model):
    """ Private message model, from user to user(s) """
    sender = models.ForeignKey(AUTH_USER_MODEL,
                               related_name='sent_discussions',
                               verbose_name=_('sender'))

    recipients = models.ManyToManyField(AUTH_USER_MODEL,
                                        through='Recipient',
                                        related_name='received_discussions',
                                        verbose_name=_('recipients'))

    objects = DiscussionManager()

    created_at = models.DateTimeField(_('created at'),
                                      auto_now_add=True)

    updated_at = models.DateTimeField(_('updated at'),
                                      null=True, blank=True)

    sender_deleted_at = models.DateTimeField(_("sender deleted at"),
                                             null=True,
                                             blank=True)

    latest_message = models.ForeignKey('Message', null=True, blank=True, related_name='latest_discussions')

    subject = models.CharField(max_length=255)

    objects = DiscussionManager()

    class Meta:
        ordering = ['-created_at']
        verbose_name = _('discussion')
        verbose_name_plural = _('discussions')
        permissions = (
            ('can_view', 'Can view'),
        )
        app_label = 'discussions'

    def __str__(self):
        return 'Discussion opened by %s' % self.sender

    def save_recipients(self, to_user_list):
        """
        Save the recipients for this message

        :param to_user_list:
            A list which elements are :class:`User` to whom the message is for.

        :return:
            Boolean indicating if any users are saved.

        """
        created = False
        for user in to_user_list:
            Recipient.objects.create(user=user,
                                     discussion=self)
            created = True
        return created

    def update_contacts(self, to_user_list):
        """
        Updates the contacts that are used for this message.

        :param to_user_list:
            List of Django :class:`User`.

        :return:
            A boolean if a user is contact is updated.

        """
        updated = False
        for user in to_user_list:
            Contact.objects.update_contact(self.sender,
                                           user,
                                           self)
            updated = True
        return updated

    def add_message(self, body, sender=None, commit=True):
        if not sender:
            sender = self.sender

        m = Message(sender=sender,
                    body=body,
                    discussion=self)
        m.save()

        self.recipient_set.exclude(user=sender).update(status=Recipient.STATUS.unread)

        self.updated_at = tznow()

        if commit:
            self.save()

        return m

    def get_absolute_url(self):
        return reverse('discussions_detail', kwargs={
            'discussion_id': self.pk
        })

    def can_view(self, user):
        if not user or not user.is_authenticated():
            return False

        if (user.is_staff or user.is_superuser or
            (user.id in [u['id']
                         for u in self.recipients.values('id')])):
            return True

        return user.has_perm('discussions.can_view')


@python_2_unicode_compatible
class Message(models.Model):
    """ Private message model, from user to user(s) """
    sender = models.ForeignKey(AUTH_USER_MODEL,
                               related_name='sent_messages',
                               verbose_name=_('sender'))

    discussion = models.ForeignKey('Discussion',
                                   related_name='messages',
                                   verbose_name=_('discussion'))

    body = models.TextField(_('body'))

    sent_at = models.DateTimeField(_('sent at'),
                                   auto_now_add=True)

    sender_deleted_at = models.DateTimeField(_('sender deleted at'),
                                             null=True,
                                             blank=True)

    objects = MessageManager()

    class Meta:
        ordering = ['-sent_at']
        verbose_name = _('message')
        verbose_name_plural = _('messages')
        app_label = 'discussions'

    def __str__(self):
        """ Human representation, displaying first ten words of the body. """
        from ..compat import truncate_words

        truncated_body = truncate_words(self.body, 10)
        return "%(truncated_body)s" % {'truncated_body': truncated_body}


@python_2_unicode_compatible
class Folder(models.Model):
    name = models.CharField(max_length=255, verbose_name=_('Name'))

    created_at = models.DateTimeField(_('created at'),
                                      auto_now_add=True)

    user = models.ForeignKey(AUTH_USER_MODEL)

    class Meta:
        ordering = ['-created_at']
        verbose_name = _('folder')
        verbose_name_plural = _('folders')
        app_label = 'discussions'

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('discussions_folder_detail', kwargs={
            'folder_id': self.pk
        })



from datetime import datetime

from django.views.generic import DetailView, FormView, ListView
from django.views.generic.base import View
from django.views.generic.edit import FormMixin, CreateView, UpdateView
from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404, redirect
from django.contrib import messages
from django.utils.translation import ungettext
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.db import models
from django.http import Http404

from ..models import Discussion, Folder, Recipient
from ..forms import ComposeForm, ReplyForm, FolderForm
from ..helpers import lookup_discussions, lookup_profiles
from .. import settings
from ..compat import User

from pure_pagination import Paginator


class DiscussionListView(ListView):
    template_name = 'discussions/list.html'
    paginate_by = settings.PAGINATE_BY
    model = Recipient
    context_object_name = 'recipient_list'
    paginator_class = Paginator

    def get_queryset(self):
        self.user = None

        if self.kwargs.get('username'):
            username = self.kwargs.get('username')

            self.user = get_object_or_404(User, username=username)

            qs = (self.model.objects
                  .filter(models.Q(user=self.request.user, discussion__sender=self.user) |
                          models.Q(user=self.user, discussion__sender=self.request.user)))
        else:
            qs = (self.model.objects
                  .filter(user=self.request.user))

        qs = (qs.exclude(status=self.model.STATUS.deleted)
              .exclude(folder__isnull=False)
              .order_by('-discussion__updated_at', '-discussion__created_at')
              .select_related('user'))

        return qs

    def get_context_data(self, **kwargs):
        context = super(DiscussionListView, self).get_context_data(**kwargs)

        lookup_discussions(context[self.context_object_name])
        lookup_profiles(context[self.context_object_name])

        return context


class DiscussionSentView(DiscussionListView):
    template_name = 'discussions/sent.html'

    def get_queryset(self):
        user = self.request.user

        qs = (self.model.objects
              .filter(discussion__sender=user, user=user))

        qs = (qs.exclude(status=self.model.STATUS.deleted)
              .order_by('-discussion__created_at')
              .select_related('user'))

        return qs


class DiscussionUnreadView(DiscussionListView):
    template_name = 'discussions/unread.html'

    def get_queryset(self):
        user = self.request.user

        qs = (self.model.objects
              .filter(user=user))

        qs = (qs.filter(status=self.model.STATUS.unread)
              .order_by('-discussion__updated_at', '-discussion__created_at')
              .select_related('user'))

        return qs


class DiscussionDeletedView(DiscussionListView):
    template_name = 'discussions/deleted.html'

    def get_queryset(self):
        user = self.request.user

        qs = (self.model.objects
              .filter(user=user))

        qs = (qs.filter(status=self.model.STATUS.deleted)
              .order_by('-discussion__created_at')
              .select_related('user'))

        return qs


class DiscussionDetailView(DetailView, FormMixin):
    pk_url_kwarg = 'discussion_id'
    model = Discussion
    context_object_name = 'discussion'
    template_name = 'discussions/detail.html'
    form_class = ReplyForm

    def get_success_url(self):
        return self.object.get_absolute_url()

    def is_allowed(self, user):
        return self.object.can_view(user)

    def get_context_data(self, **kwargs):
        data = super(DiscussionDetailView, self).get_context_data(**kwargs)

        if not self.is_allowed(self.request.user):
            raise Http404

        recipients = Recipient.objects.filter(discussion=self.object,
                                              user=self.request.user)

        now = datetime.now()
        recipients.update(read_at=now, status=Recipient.STATUS.read)

        users = self.object.recipients.select_related('user')

        return dict(data, **{
            'recipients': recipients,
            'users': users,
            'form': self.get_form(self.get_form_class()),
            'message_list': self.get_messages(self.object)
        })

    def get_messages(self, discussion):
        return discussion.messages.order_by('sent_at')

    def get_template_names(self):
        return [self.template_name]

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()

        form_class = self.get_form_class()
        form = self.get_form(form_class)

        if form.is_valid():
            return self.form_valid(form)

        return self.form_invalid(form)

    def form_valid(self, form):
        form.save(self.request.user)

        return super(DiscussionDetailView, self).form_valid(form)

    def get_form_kwargs(self):
        return dict(super(DiscussionDetailView, self).get_form_kwargs(), **{
            'discussion': self.get_object()
        })


class MessageComposeView(FormView):
    form_class = ComposeForm
    template_name = 'discussions/form.html'
    submit_key = 'submit'

    def get_template_names(self):
        return [self.template_name, ]

    def get_initial(self):
        recipients = None
        initial = {}

        if self.request.method == 'POST':
            recipients = self.request.POST.get('recipients', None)
        elif self.kwargs.get('recipients'):
            recipients = self.kwargs.get('recipients')

        if recipients:
            username_list = [r.strip() for r in recipients.split('+')]
            recipients = [u for u in User.objects.filter(username__in=username_list)]

            self.recipients = recipients

            initial['to'] = recipients

        return initial.copy()

    def get_context_data(self, **kwargs):
        data = super(MessageComposeView, self).get_context_data(**kwargs)

        return dict(data, **{'recipients': self.recipients
                             if hasattr(self, 'recipients') else None})

    def form_valid(self, form):
        self.object = form.save(self.request.user)

        return redirect(self.get_success_url(form))

    def get_success_url(self, form):
        requested_redirect = self.request.REQUEST.get(REDIRECT_FIELD_NAME,
                                                      False)

        # Redirect mechanism
        redirect_to = reverse('discussions_list')
        if requested_redirect:
            redirect_to = requested_redirect
        elif self.kwargs.get('success_url'):
            redirect_to = self.kwargs.get('success_url')
        elif len(form.cleaned_data['to']) == 1:
            redirect_to = reverse('discussions_detail',
                                  kwargs={'discussion_id': self.object.pk})

        return redirect_to

    def post(self, request, *args, **kwargs):
        if self.submit_key in request.POST:
            return super(MessageComposeView, self).post(request, *args, **kwargs)

        return self.get(request, *args, **kwargs)


class DiscussionBulkMixin(object):
    def valid_ids(self, ids):
        valid_discussion_id_list = set()
        for pk in ids:
            try:
                valid_pk = int(pk)
            except (TypeError, ValueError):
                pass
            else:
                valid_discussion_id_list.add(valid_pk)

        return valid_discussion_id_list


class DiscussionMoveView(DetailView, DiscussionBulkMixin):
    http_method_names = ['post']
    model = Folder
    pk_url_kwarg = 'folder_id'

    def get_queryset(self):
        return self.model.objects.filter(user=self.request.user)

    def post(self, *args, **kwargs):
        """
        A ``POST`` to move discussions into a folder.

        POST can have the following keys:

            ``discussions_ids``
                List of discussion id's that should be moved.
        """

        self.object = self.get_object()
        self.get_context_data(object=self.object)

        discussion_ids = self.request.POST.getlist('discussion_ids')

        if discussion_ids:
            Recipient.objects.filter(discussion__in=self.valid_ids(discussion_ids),
                                     user=self.request.user).update(folder=self.object)

        return redirect(reverse('discussions_list'))


class DiscussionReadView(View, DiscussionBulkMixin):
    http_method_names = ['post']

    def post(self, *args, **kwargs):
        """
        A ``POST`` to mark as read discussions.

        POST can have the following keys:

            ``discussions_ids``
                List of discussion id's that should be marked as read.
        """

        discussion_ids = self.request.POST.getlist('discussion_ids')

        if discussion_ids:
            recipients = (Recipient.objects.filter(discussion__in=self.valid_ids(discussion_ids),
                                                   user=self.request.user)
                          .exclude(status=Recipient.STATUS.read))
            for recipient in recipients:
                recipient.mark_as_read()

        return redirect(reverse('discussions_list'))


class DiscussionRemoveView(View, DiscussionBulkMixin):
    http_method_names = ['post']

    def post(self, *args, **kwargs):
        """
        A ``POST`` to remove discussions.

        :param undo:
            A Boolean that if ``True`` unremoves discussions.

        POST can have the following keys:

            ``discussions_ids``
                List of discussion id's that should be deleted.

            ``next``
                String containing the URI which to redirect to after the keys are
                removed. Redirect defaults to the inbox view.

        The ``next`` value can also be supplied in the URI with ``?next=<value>``.

        """
        discussion_ids = self.request.POST.getlist('discussion_ids')
        redirect_to = self.request.REQUEST.get('next', False)
        undo = self.kwargs.get('undo', False)

        if discussion_ids:
            # Delete all the messages, if they belong to the user.
            now = datetime.now()
            changed_message_list = set()
            for pk in self.valid_ids(discussion_ids):
                discussion = get_object_or_404(Discussion, pk=pk)

                # Check if the user is the owner
                if discussion.sender_id == self.request.user.pk:
                    if undo:
                        discussion.sender_deleted_at = None
                    else:
                        discussion.sender_deleted_at = now
                    discussion.save()
                    changed_message_list.add(discussion.pk)

                # Check if the user is a recipient of the message
                recipients = discussion.recipient_set.filter(user=self.request.user,
                                                             discussion=discussion)

                if recipients:
                    for recipient in recipients:
                        if undo:
                            recipient.mark_as_read()
                        else:
                            recipient.mark_as_deleted()

                    changed_message_list.add(discussion.pk)

            # Send messages
            if (len(changed_message_list) > 0):
                if undo:
                    message = ungettext('Discussion is succesfully restored.',
                                        'Discussions are succesfully restored.',
                                        len(changed_message_list))
                else:
                    message = ungettext('Discussion is successfully removed.',
                                        'Discussions are successfully removed.',
                                        len(changed_message_list))

                messages.success(self.request, message, fail_silently=True)

        if redirect_to:
            return redirect(redirect_to)

        return redirect(reverse('discussions_list'))


class FolderCreateView(CreateView):
    form_class = FolderForm
    template_name = 'discussions/folder/create.html'

    def form_valid(self, form):
        self.object = form.save(self.request.user)

        return redirect(self.get_success_url())

    def get_success_url(self):
        return reverse('discussions_folder_detail', kwargs={
            'folder_id': self.object.pk
        })


class FolderUpdateView(UpdateView):
    form_class = FolderForm
    model = Folder
    template_name = 'discussions/folder/update.html'
    pk_url_kwarg = 'folder_id'

    def get_queryset(self):
        return self.model.objects.filter(pk=self.kwargs.get(self.pk_url_kwarg),
                                         user=self.request.user)

    def form_valid(self, form):
        self.object = form.save(self.request.user)

        return redirect(self.get_success_url())

    def get_success_url(self):
        return reverse('discussions_folder_update', kwargs={
            'folder_id': self.object.pk
        })


class FolderDetailView(DiscussionListView):
    model = Folder
    template_name = 'discussions/folder/detail.html'
    context_object_name = 'recipient_list'
    context_object = 'folder'
    paginate_by = settings.PAGINATE_BY

    def get_object(self):
        obj = get_object_or_404(Folder.objects.filter(user=self.request.user),
                                pk=self.kwargs.get('folder_id'))

        self.object = obj

        return obj

    def get_context_data(self, **kwargs):
        data = super(FolderDetailView, self).get_context_data(**kwargs)

        data[self.context_object] = self.object

        return data

    def get_queryset(self):
        return (self.get_object()
                .recipients
                .order_by('-discussion__created_at')
                .select_related('user'))



import django


# Django 1.5+ compatibility
if django.VERSION >= (1, 5):
    from django.contrib.auth import get_user_model
    User = get_user_model()

    update_fields = lambda instance, fields: instance.save(update_fields=fields)
else:
    from django.contrib.auth.models import User

    update_fields = lambda instance, fields: instance.save()

# Django 1.6+ compatibility
if django.VERSION >= (1, 6):
    from django.utils.text import Truncator

    def truncate_words(s, num, end_text='...'):
        truncate = end_text and ' %s' % end_text or ''
        return Truncator(s).words(num, truncate=truncate)
else:
    from django.utils.text import truncate_words

__all__ = ['User', 'truncate_words']



from discussions.utils import load_class

from discussions import settings

DiscussionListView = load_class(settings.DISCUSSION_LIST_VIEW)

DiscussionSentView = load_class(settings.DISCUSSION_SENT_VIEW)

DiscussionDeletedView = load_class(settings.DISCUSSION_DELETED_VIEW)

DiscussionUnreadView = load_class(settings.DISCUSSION_UNREAD_VIEW)

DiscussionDetailView = load_class(settings.DISCUSSION_DETAIL_VIEW)

DiscussionRemoveView = load_class(settings.DISCUSSION_REMOVE_VIEW)

DiscussionMoveView = load_class(settings.DISCUSSION_MOVE_VIEW)

DiscussionReadView = load_class(settings.DISCUSSION_READ_VIEW)

MessageComposeView = load_class(settings.MESSAGE_COMPOSE_VIEW)

FolderCreateView = load_class(settings.FOLDER_CREATE_VIEW)

FolderUpdateView = load_class(settings.FOLDER_UPDATE_VIEW)

FolderDetailView = load_class(settings.FOLDER_DETAIL_VIEW)



from __future__ import unicode_literals

from django.test import TestCase
from django.template import Context, Template

from ..compat import User


class TemplateTagsTests(TestCase):
    fixtures = ['users.json', 'messages.json']

    def test_get_unread_message_count_for(self):
        user = User.objects.get(pk=1)

        t = Template("{% load discussions_tags %}{% get_unread_message_count_for user as message_count %}{{ message_count }}")
        c = Context({
            'user': user
        })
        self.failUnlessEqual('1', t.render(c))

    def test_get_unread_message_count_between(self):
        thoas = User.objects.get(pk=1)
        ampelmann = User.objects.get(pk=2)

        t = Template("{% load discussions_tags %}{% get_unread_message_count_between user_1 and user_2 as message_count %}{{ message_count }}")
        c = Context({
            'user_1': thoas,
            'user_2': ampelmann
        })
        self.failUnlessEqual('0', t.render(c))

    def test_get_folders_for(self):
        user = User.objects.get(pk=1)

        t = Template("{% load discussions_tags %}{% get_folders_for user as folders %}{% for folder in folders %}{{ folder.name }}{% endfor %}")
        c = Context({
            'user': user
        })

        self.failUnlessEqual('My folder', t.render(c))

        c = Context({
            'user': User.objects.get(pk=2)
        })

        self.failUnlessEqual('', t.render(c))  # return nothing



from django import template

from ..models import Recipient, Folder

import re

register = template.Library()


class Folders(template.Node):
    def __init__(self, user, var_name):
        self.user = template.Variable(user)
        self.var_name = var_name

    def render(self, context):
        try:
            user = self.user.resolve(context)
        except template.VariableDoesNotExist:
            return ''

        folders = Folder.objects.filter(user=user)

        if self.var_name:
            context[self.var_name] = folders

        return ''


class MessageCount(template.Node):
    def __init__(self, from_user, var_name, to_user=None):
        self.user = template.Variable(from_user)
        self.var_name = var_name
        if to_user:
            self.to_user = template.Variable(to_user)
        else:
            self.to_user = to_user

    def render(self, context):
        try:
            user = self.user.resolve(context)
        except template.VariableDoesNotExist:
            return ''

        if not self.to_user:
            message_count = Recipient.objects.count_unread_messages_for(user)

        else:
            try:
                to_user = self.to_user.resolve(context)
            except template.VariableDoesNotExist:
                return ''

            message_count = Recipient.objects.count_unread_messages_between(user,
                                                                            to_user)

        context[self.var_name] = message_count

        return ''


@register.tag
def get_folders_for(parser, token):
    """
    Returns the folders for a user

    Syntax::

        {% get_folders_for [user] as [var_name] %}

    Example usage::

        {% get_folders_for pero as folders %}

    """
    try:
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError, "%s tag requires arguments" % token.contents.split()[0]
    m = re.search(r'(.*?) as (\w+)', arg)

    if not m:
        raise template.TemplateSyntaxError, "%s tag had invalid arguments" % tag_name
    user, var_name = m.groups()
    return Folders(user, var_name)


@register.tag
def get_unread_message_count_for(parser, token):
    """
    Returns the unread message count for a user.

    Syntax::

        {% get_unread_message_count_for [user] as [var_name] %}

    Example usage::

        {% get_unread_message_count_for pero as message_count %}

    """
    try:
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError, "%s tag requires arguments" % token.contents.split()[0]
    m = re.search(r'(.*?) as (\w+)', arg)
    if not m:
        raise template.TemplateSyntaxError, "%s tag had invalid arguments" % tag_name
    user, var_name = m.groups()
    return MessageCount(user, var_name)


@register.tag
def get_unread_message_count_between(parser, token):
    """
    Returns the unread message count between two users.

    Syntax::

        {% get_unread_message_count_between [user] and [user] as [var_name] %}

    Example usage::

        {% get_unread_message_count_between funky and wunki as message_count %}

    """
    try:
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError, "%s tag requires arguments" % token.contents.split()[0]
    m = re.search(r'(.*?) and (.*?) as (\w+)', arg)
    if not m:
        raise template.TemplateSyntaxError, "%s tag had invalid arguments" % tag_name
    from_user, to_user, var_name = m.groups()
    return MessageCount(from_user, var_name, to_user)



from discussions import settings

from discussions.utils import load_class

ComposeForm = load_class(settings.COMPOSE_FORM)

ReplyForm = load_class(settings.REPLY_FORM)

FolderForm = load_class(settings.FOLDER_FORM)



from django.test import TestCase
from django.core.urlresolvers import reverse

from ..forms import ComposeForm, FolderForm
from ..models import Message, Recipient, Discussion, Folder
from ..compat import User


class DiscussionsViewsTests(TestCase):
    fixtures = ['users.json', 'messages.json']

    def _test_login(self, named_url, **kwargs):
        """ Test that the view requires login """
        response = self.client.get(reverse(named_url, **kwargs))
        self.assertEqual(response.status_code, 302)

    def test_compose(self):
        """ A ``GET`` to the compose view """
        # Login is required.
        self._test_login('discussions_compose')

        # Sign in
        self.client.login(username='thoas', password='$ecret')
        response = self.client.get(reverse('discussions_compose'))

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response,
                                'discussions/form.html')

        self.failUnless(isinstance(response.context['form'],
                                   ComposeForm))

    def test_compose_post(self):
        """ ``POST`` to the compose view """
        self.client.login(username='thoas', password='$ecret')

        valid_data = {'to': 'ampelmann',
                      'body': 'Hi mister',
                      'subject': 'Hi'}

        response = self.client.post(reverse('discussions_compose'),
                                    data=valid_data)

        self.assertEqual(response.status_code, 200)

        valid_data['submit'] = 'submit'

        response = self.client.post(reverse('discussions_compose'),
                                    data=valid_data)

        self.assertRedirects(response,
                             reverse('discussions_detail',
                                     kwargs={'discussion_id': 3}))

        # Check for a requested redirect
        valid_data['next'] = reverse('discussions_compose')
        response = self.client.post(reverse('discussions_compose'),
                                    data=valid_data)
        self.assertRedirects(response,
                             valid_data['next'])

    def test_compose_recipients_get(self):
        """ A ``GET`` to the compose view with recipients """
        self.client.login(username='thoas', password='$ecret')

        valid_recipients = "thoas+oleiade"

        # Test valid recipients
        response = self.client.get(reverse('discussions_compose',
                                           kwargs={'recipients': valid_recipients}))

        self.assertEqual(response.status_code, 200)

        # Test the users
        oleiade = User.objects.get(username='oleiade')
        thoas = User.objects.get(username='thoas')
        self.assertEqual(response.context['recipients'][0], oleiade)
        self.assertEqual(response.context['recipients'][1], thoas)

        # Test that the initial data of the form is set.
        self.assertEqual(response.context['form'].initial['to'],
                         [oleiade, thoas])

    def test_compose_recipients_post(self):
        """ A ``POST`` to the compose view with recipients """
        self.client.login(username='thoas', password='$ecret')

        valid_recipients = "thoas+oleiade"

        # Test valid recipients
        response = self.client.post(reverse('discussions_compose'), data={
            'recipients': valid_recipients
        })

        self.assertEqual(response.status_code, 200)

        # Test the users
        oleiade = User.objects.get(username='oleiade')
        thoas = User.objects.get(username='thoas')
        self.assertEqual(response.context['recipients'][0], oleiade)
        self.assertEqual(response.context['recipients'][1], thoas)

        # Test that the initial data of the form is set.
        self.assertEqual(response.context['form'].initial['to'],
                         [oleiade, thoas])

    def test_discussion_detail(self):
        """ A ``GET`` to the detail view """
        self._test_login('discussions_detail',
                         kwargs={'discussion_id': 2})

        # Sign in
        self.client.login(username='ampelmann', password='$ecret')
        response = self.client.get(reverse('discussions_detail',
                                   kwargs={'discussion_id': 1}))

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response,
                                'discussions/detail.html')

        # Test that the message is read.
        ampelmann = User.objects.get(pk=2)
        mr = Recipient.objects.get(discussion=Discussion.objects.get(pk=1),
                                   user=ampelmann)
        self.failUnless(mr.read_at)

    def test_discussion_detail_new_message(self):
        self.client.login(username='ampelmann', password='$ecret')
        discussion = Discussion.objects.get(pk=1)

        response = self.client.post(discussion.get_absolute_url(),
                                    data={
                                        'body': 'My reply'
                                    })
        self.assertRedirects(response, discussion.get_absolute_url())

        self.assertEqual(discussion.messages.count(), 2)

        ampelmann = User.objects.get(username='ampelmann')

        message = discussion.messages.get(sender=ampelmann)

        self.assertEqual(message.body, 'My reply')

    def test_valid_discussion_move(self):
        """ ``POST`` to move a discussion into a folder """
        # Test that sign in is required

        folder = Folder.objects.get(pk=1)

        url = reverse('discussions_move', kwargs={
            'folder_id': folder.pk
        })
        response = self.client.post(url)
        self.assertEqual(response.status_code, 302)

        # Sign in
        self.client.login(username='thoas', password='$ecret')

        # Test that only posts are allowed
        response = self.client.get(url)
        self.assertEqual(response.status_code, 405)

        response = self.client.post(url,
                                    data={'discussion_ids': '1'})
        self.assertRedirects(response,
                             reverse('discussions_list'))

        user = User.objects.get(username='thoas')

        recipient = Recipient.objects.get(discussion=1, user=user)

        self.assertEqual(recipient.folder, folder)

    def test_invalid_discussion_move(self):
        folder = Folder.objects.get(pk=1)

        url = reverse('discussions_move', kwargs={
            'folder_id': folder.pk
        })

        self.client.login(username='ampelmann', password='$ecret')

        response = self.client.post(url,
                                    data={'discussion_ids': '1'})

        self.assertEqual(response.status_code, 404)

    def test_valid_discussion_read(self):
        """ ``POST`` to remove a discussion """
        # Test that sign in is required
        response = self.client.post(reverse('discussions_read'))
        self.assertEqual(response.status_code, 302)

        # Sign in
        self.client.login(username='thoas', password='$ecret')

        # Test that only posts are allowed
        response = self.client.get(reverse('discussions_read'))
        self.assertEqual(response.status_code, 405)

        response = self.client.post(reverse('discussions_read'),
                                    data={'discussion_ids': '1'})
        self.assertRedirects(response,
                             reverse('discussions_list'))

        recipient = Discussion.objects.get(pk=1).recipient_set.get(user=User.objects.get(pk=1))

        self.assertTrue(recipient.is_read())

    def test_valid_discussion_remove(self):
        """ ``POST`` to remove a discussion """
        # Test that sign in is required
        response = self.client.post(reverse('discussions_remove'))
        self.assertEqual(response.status_code, 302)

        # Sign in
        self.client.login(username='thoas', password='$ecret')

        # Test that only posts are allowed
        response = self.client.get(reverse('discussions_remove'))
        self.assertEqual(response.status_code, 405)

        response = self.client.post(reverse('discussions_remove'),
                                    data={'discussion_ids': '1'})
        self.assertRedirects(response,
                             reverse('discussions_list'))
        d = Discussion.objects.get(pk=1)
        self.failUnless(d.sender_deleted_at)

        self.client.login(username='ampelmann', password='$ecret')
        response = self.client.post(reverse('discussions_remove'),
                                    data={'discussion_ids': '1',
                                          'next': reverse('discussions_list')})
        self.assertRedirects(response,
                             reverse('discussions_list'))
        ampelmann = User.objects.get(username='ampelmann')
        dr = d.recipient_set.get(user=ampelmann,
                                 discussion=d)
        self.failUnless(dr.deleted_at)

    def test_invalid_discussion_remove(self):
        """ ``POST`` to remove an invalid discussion """
        # Sign in
        self.client.login(username='thoas', password='$ecret')

        bef_len = Message.objects.filter(sender_deleted_at__isnull=False).count()
        response = self.client.post(reverse('discussions_remove'),
                                    data={'discussion_ids': ['a', 'b']})

        # The program should play nice, nothing happened.
        af_len = Message.objects.filter(sender_deleted_at__isnull=False).count()
        self.assertRedirects(response,
                             reverse('discussions_list'))
        self.assertEqual(bef_len, af_len)

    def test_valid_discussion_remove_multiple(self):
        """ ``POST`` to remove multiple messages """
        # Sign in
        self.client.login(username='thoas', password='$ecret')
        response = self.client.post(reverse('discussions_remove'),
                                    data={'discussion_ids': [1, 2]})
        self.assertRedirects(response,
                             reverse('discussions_list'))

        discussion_list = Discussion.objects.filter(pk__in=['1', '2'],
                                                    sender_deleted_at__isnull=False)

        # thoas has created only one discussion (see fixtures)
        self.assertEqual(discussion_list.count(), 1)

    def test_discussion_unremove(self):
        """ Unremove a discussion """
        self.client.login(username='thoas', password='$ecret')

        # Delete a discussion as owner
        response = self.client.post(reverse('discussions_unremove'),
                                    data={'discussion_pks': [1, ]})

        self.assertRedirects(response,
                             reverse('discussions_list'))

        # Delete the message as a recipient
        response = self.client.post(reverse('discussions_unremove'),
                                    data={'discussion_pks': [2, ]})

        self.assertRedirects(response,
                             reverse('discussions_list'))

    def test_discussion_list(self):
        """ ``GET`` the discussion list for a user """
        self._test_login("discussions_list")

        self.client.login(username='thoas', password='$ecret')
        response = self.client.get(reverse('discussions_list'))
        self.assertEqual(response.status_code, 200)

        self.assertTemplateUsed(response, 'discussions/list.html')

    def test_discussion_sent(self):
        """ ``GET`` the message list for a user """
        self._test_login("discussions_sent")

        self.client.login(username='thoas', password='$ecret')
        response = self.client.get(reverse('discussions_sent'))
        self.assertEqual(response.status_code, 200)

        self.assertTemplateUsed(response, 'discussions/sent.html')

    def test_discussion_deleted(self):
        """ ``GET`` the message list for a user """
        self._test_login("discussions_deleted")

        self.client.login(username='thoas', password='$ecret')
        response = self.client.get(reverse('discussions_deleted'))
        self.assertEqual(response.status_code, 200)

        self.assertTemplateUsed(response, 'discussions/deleted.html')

    def test_discussion_unread(self):
        """ ``GET`` the message list for a user """
        self._test_login("discussions_unread")

        self.client.login(username='thoas', password='$ecret')
        response = self.client.get(reverse('discussions_unread'))
        self.assertEqual(response.status_code, 200)

        self.assertTemplateUsed(response, 'discussions/unread.html')

    def test_discussion_detail_between_two_users(self):
        """ ``GET`` to a detail page between two users """
        self._test_login('discussions_list',
                         kwargs={'username': 'oleiade'})
        self.client.login(username='thoas', password='$ecret')

        response = self.client.get(reverse('discussions_list',
                                           kwargs={'username': 'oleiade'}))

        self.assertEqual(response.status_code, 200)

        self.assertTemplateUsed(response, 'discussions/list.html')

        # Check that all the messages are marked as read.
        thoas = User.objects.get(pk=1)
        oleiade = User.objects.get(pk=3)
        unread_messages = Recipient.objects.filter(user=thoas,
                                                   discussion__sender=oleiade,
                                                   read_at__isnull=True)

        self.assertEqual(len(unread_messages), 0)

    def test_folder_create(self):
        self._test_login('discussions_folder_create')

        self.client.login(username='thoas', password='$ecret')

        response = self.client.get(reverse('discussions_folder_create'))

        self.assertEqual(response.status_code, 200)

        self.failUnless(isinstance(response.context['form'], FolderForm))
        self.assertTemplateUsed(response,
                                'discussions/folder/create.html')

        data = {
            'name': 'My folder'
        }

        response = self.client.post(reverse('discussions_folder_create'), data=data)

        self.assertEqual(Folder.objects.count(), 2)

        folder = Folder.objects.get(pk=2)

        self.assertEqual(folder.name, data['name'])

        self.assertRedirects(response,
                             reverse('discussions_folder_detail', kwargs={
                                 'folder_id': folder.pk
                             }))

    def test_folder_update(self):
        folder = Folder.objects.get(pk=1)

        self._test_login('discussions_folder_update', kwargs={
            'folder_id': folder.pk
        })

        self.client.login(username='thoas', password='$ecret')

        response = self.client.get(reverse('discussions_folder_update', kwargs={
            'folder_id': folder.pk
        }))

        self.assertEqual(response.status_code, 200)

        self.failUnless(isinstance(response.context['form'], FolderForm))
        self.assertTemplateUsed(response,
                                'discussions/folder/update.html')

        data = {
            'name': 'My folder new name'
        }

        response = self.client.post(reverse('discussions_folder_update', kwargs={
            'folder_id': folder.pk
        }), data=data)

        folder = Folder.objects.get(pk=1)

        self.assertEqual(folder.name, data['name'])

        self.assertRedirects(response,
                             reverse('discussions_folder_update', kwargs={
                                 'folder_id': folder.pk
                             }))



from django.test import TestCase
from django import forms

from ..fields import CommaSeparatedUserField


class CommaSeparatedTestForm(forms.Form):
    users = CommaSeparatedUserField()

    def __init__(self, *args, **kwargs):
        super(CommaSeparatedTestForm, self).__init__(*args, **kwargs)
        self.fields['users']._recipient_filter = self.filter_jane

    def filter_jane(self, user):
        if user.username == 'ampelmann':
            return False
        return True


class CommaSeperatedFieldTests(TestCase):
    fixtures = ['users.json']

    def test_invalid_data(self):
        # Test invalid data supplied to the field.
        invalid_data_dicts = [
            # Empty username
            {'data': {'users': ''},
             'error': ('users', [u'This field is required.'])},
            # No data
            {'data': {},
             'error': ('users', [u'This field is required.'])},
            # A list
            {'data': {'users': []},
             'error': ('users', [u'This field is required.'])},
            # Forbidden username
            {'data': {'users': 'ampelmann'},
             'error': ('users', [u'The following usernames are incorrect: ampelmann.'])},
            # Non-existant username
            {'data': {'users': 'foo'},
             'error': ('users', [u'The following usernames are incorrect: foo.'])},
            # Multiple invalid usernames
            {'data': {'users': 'foo, bar'},
             'error': ('users', [u'The following usernames are incorrect: foo, bar.'])},
            # Valid and invalid
            {'data': {'users': 'foo, thoas, bar'},
             'error': ('users', [u'The following usernames are incorrect: foo, bar.'])},
            # Extra whitespace
            {'data': {'users': 'foo,    thoas  '},
             'error': ('users', [u'The following usernames are incorrect: foo.'])},

        ]
        for invalid_dict in invalid_data_dicts:
            form = CommaSeparatedTestForm(data=invalid_dict['data'])
            self.failIf(form.is_valid())
            self.assertEqual(form.errors[invalid_dict['error'][0]],
                             invalid_dict['error'][1])



from django.conf import settings


PAGINATE_BY = getattr(settings, 'DISCUSSIONS_PAGINATE_BY', 20)

RECIPIENT_MODEL = getattr(settings,
                          'DISCUSSIONS_RECIPIENT_MODEL',
                          'discussions.models.base.Recipient')
DISCUSSION_MODEL = getattr(settings,
                           'DISCUSSIONS_DISCUSSION_MODEL',
                           'discussions.models.base.Discussion')
FOLDER_MODEL = getattr(settings,
                       'DISCUSSIONS_FOLDER_MODEL',
                       'discussions.models.base.Folder')
MESSAGE_MODEL = getattr(settings,
                        'DISCUSSIONS_MESSAGE_MODEL',
                        'discussions.models.base.Message')
CONTACT_MODEL = getattr(settings,
                        'DISCUSSIONS_CONTACT_MODEL',
                        'discussions.models.base.Contact')

DISCUSSION_LIST_VIEW = getattr(settings,
                               'DISCUSSIONS_DISCUSSION_LIST_VIEW',
                               'discussions.views.base.DiscussionListView')
DISCUSSION_SENT_VIEW = getattr(settings,
                               'DISCUSSIONS_DISCUSSION_SENT_VIEW',
                               'discussions.views.base.DiscussionSentView')
DISCUSSION_DELETED_VIEW = getattr(settings,
                                  'DISCUSSIONS_DISCUSSION_DELETED_VIEW',
                                  'discussions.views.base.DiscussionDeletedView')
DISCUSSION_UNREAD_VIEW = getattr(settings,
                                 'DISCUSSIONS_DISCUSSION_UNREAD_VIEW',
                                 'discussions.views.base.DiscussionUnreadView')
DISCUSSION_DETAIL_VIEW = getattr(settings,
                                 'DISCUSSIONS_DISCUSSION_DETAIL_VIEW',
                                 'discussions.views.base.DiscussionDetailView')
DISCUSSION_REMOVE_VIEW = getattr(settings,
                                 'DISCUSSIONS_DISCUSSION_REMOVE_VIEW',
                                 'discussions.views.base.DiscussionRemoveView')
DISCUSSION_READ_VIEW = getattr(settings,
                               'DISCUSSIONS_DISCUSSION_READ_VIEW',
                               'discussions.views.base.DiscussionReadView')
DISCUSSION_MOVE_VIEW = getattr(settings,
                               'DISCUSSIONS_DISCUSSION_MOVE_VIEW',
                               'discussions.views.base.DiscussionMoveView')
MESSAGE_COMPOSE_VIEW = getattr(settings,
                               'DISCUSSIONS_MESSAGE_COMPOSE_VIEW',
                               'discussions.views.base.MessageComposeView')
FOLDER_CREATE_VIEW = getattr(settings,
                             'DISCUSSIONS_FOLDER_CREATE_VIEW',
                             'discussions.views.base.FolderCreateView')
FOLDER_UPDATE_VIEW = getattr(settings,
                             'DISCUSSIONS_FOLDER_UPDATE_VIEW',
                             'discussions.views.base.FolderUpdateView')
FOLDER_DETAIL_VIEW = getattr(settings,
                             'DISCUSSIONS_FOLDER_DETAIL_VIEW',
                             'discussions.views.base.FolderDetailView')

COMPOSE_FORM = getattr(settings, 'DISCUSSIONS_COMPOSE_FORM', 'discussions.forms.base.ComposeForm')

REPLY_FORM = getattr(settings, 'DISCUSSIONS_REPLY_FORM', 'discussions.forms.base.ReplyForm')

FOLDER_FORM = getattr(settings, 'DISCUSSIONS_FOLDER_FORM', 'discussions.forms.base.FolderForm')

PRE_FILTER = getattr(settings, 'DISCUSSIONS_PRE_FILTER', 'django.contrib.auth.decorators.login_required')



from django.db import models
from django.db.models import Q
from django.db.models import signals


class ContactManager(models.Manager):
    """ Manager for the :class:`MessageContact` model """

    def get_or_create(self, from_user, to_user, discussion):
        """
        Get or create a Contact

        We override Django's :func:`get_or_create` because we want contact to
        be unique in a bi-directional manner.

        """
        created = False
        try:
            contact = self.get(Q(from_user=from_user, to_user=to_user) |
                               Q(from_user=to_user, to_user=from_user))

        except self.model.DoesNotExist:
            created = True
            contact = self.create(from_user=from_user,
                                  to_user=to_user,
                                  latest_discussion=discussion)

        return (contact, created)

    def update_contact(self, from_user, to_user, message):
        """ Get or update a contacts information """
        contact, created = self.get_or_create(from_user,
                                              to_user,
                                              message)

        # If the contact already existed, update the message
        if not created:
            contact.latest_message = message
            contact.save()
        return contact

    def get_contacts_for(self, user):
        """
        Returns the contacts for this user.

        Contacts are other users that this user has received messages
        from or send messages to.

        :param user:
            The :class:`User` which to get the contacts for.

        """
        return self.filter(Q(from_user=user) | Q(to_user=user))


class DiscussionManager(models.Manager):
    """ Manager for the :class:`Message` model. """

    def send_message(self, sender, to_user_list, subject, body):
        """
        Send a message from a user, to a user.

        :param sender:
            The :class:`User` which sends the message.

        :param to_user_list:
            A list which elements are :class:`User` to whom the message is for.

        :param message:
            String containing the message.
        :return:
            A Discussion :class:`Discussion`

        """
        discussion = self.model(sender=sender,
                                subject=subject)
        discussion.save()

        discussion.save_recipients(list(to_user_list) + [sender, ])

        # Save the recipients
        discussion.update_contacts(to_user_list)

        discussion.add_message(body)

        return discussion

    def get_conversation_between(self, from_user, to_user):
        """ Returns a conversation between two users """
        messages = self.filter(Q(sender=from_user, recipients=to_user,
                                 sender_deleted_at__isnull=True) |
                               Q(sender=to_user, recipients=from_user,
                                 messagerecipient__deleted_at__isnull=True))
        return messages


class RecipientManager(models.Manager):
    """ Manager for the :class:`MessageRecipient` model. """

    def count_unread_messages_for(self, user):
        """
        Returns the amount of unread messages for this user

        :param user:
            A Django :class:`User`

        :return:
            An integer with the amount of unread messages.

        """
        unread_total = self.filter(user=user,
                                   status=self.model.STATUS.unread).count()

        return unread_total

    def count_unread_messages_between(self, to_user, from_user):
        """
        Returns the amount of unread messages between two users

        :param to_user:
            A Django :class:`User` for who the messages are for.

        :param from_user:
            A Django :class:`User` from whom the messages originate from.

        :return:
            An integer with the amount of unread messages.

        """
        unread_total = self.filter(discussion__sender=from_user,
                                   user=to_user,
                                   status=self.model.STATUS.unread).count()

        return unread_total


class MessageManager(models.Manager):
    def contribute_to_class(self, cls, name):
        signals.post_save.connect(self.post_save, sender=cls)
        return super(MessageManager, self).contribute_to_class(cls, name)

    def post_save(self, instance, **kwargs):
        if kwargs.get('created', False):
            discussion = instance.discussion

            discussion.latest_message = instance
            discussion.save()



from __future__ import unicode_literals

from django.test import TestCase

from ..forms import ComposeForm, ReplyForm, FolderForm
from ..models import Discussion
from ..compat import User


class ComposeFormTests(TestCase):
    """ Test the compose form. """
    fixtures = ['users.json']

    def test_invalid_data(self):
        """
        Test the save method of :class:`ComposeForm`

        We don't need to make the ``to`` field sweat because we have done that
        in the ``fields`` test.

        """
        invalid_data_dicts = [
            # No body
            {'data': {'to': 'john',
                      'body': '',
                      'subject': ''},
             'error': ('body', ['This field is required.'])},
        ]

        for invalid_dict in invalid_data_dicts:
            form = ComposeForm(data=invalid_dict['data'])
            self.failIf(form.is_valid())
            self.assertEqual(form.errors[invalid_dict['error'][0]],
                             invalid_dict['error'][1])

    def test_save_discussion(self):
        """ Test valid data """
        valid_data = {'to': 'thoas,ampelmann',
                      'body': 'Body',
                      'subject': 'subject'}

        form = ComposeForm(data=valid_data)

        self.failUnless(form.is_valid())

        # Save the form.
        sender = User.objects.get(username='oleiade')
        discussion = form.save(sender)

        # Check if the values are set correctly
        self.failUnlessEqual(discussion.subject, valid_data['subject'])

        self.assertEqual(discussion.messages.count(), 1)

        self.failUnlessEqual(discussion.sender, sender)
        self.failUnless(discussion.created_at)

        message = discussion.messages.all()[0]

        self.assertEqual(message.body, valid_data['body'])

        # Check recipients
        self.failUnlessEqual(discussion.recipients.all()[0].username, 'ampelmann')
        self.failUnlessEqual(discussion.recipients.all()[1].username, 'thoas')


class ReplyFormTests(TestCase):
    fixtures = ['users.json', 'messages.json']

    def test_save_message(self):
        valid_data = {'body': 'Body'}

        discussion = Discussion.objects.get(pk=1)

        form = ReplyForm(discussion=discussion, data=valid_data)

        self.failUnless(form.is_valid())

        sender = User.objects.get(username='oleiade')

        message = form.save(sender)

        self.failUnlessEqual(message.body, valid_data['body'])

        self.assertEqual(discussion.messages.count(), 2)

        self.assertEqual(message.sender, sender)


class FolderFormTests(TestCase):
    fixtures = ['users.json']

    def test_save_folder(self):
        valid_data = {'name': 'My folder'}

        form = FolderForm(data=valid_data)

        self.failUnless(form.is_valid())

        sender = User.objects.get(username='oleiade')

        folder = form.save(sender)

        self.failUnlessEqual(folder.name, valid_data['name'])



import os
import django

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'test_db',
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
    }
}

SITE_ID = 1
DEBUG = True

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'discussions',
]

ROOT_URLCONF = 'discussions.urls'

TEMPLATE_DIRS = (
    os.path.join(os.path.dirname(__file__), 'templates'),
)

SECRET_KEY = 'blabla'

if django.VERSION <= (1, 6):
    TEST_RUNNER = 'discover_runner.DiscoverRunner'
